Package: flowViz
Type: Package
Title: Visualization for flow cytometry
Version: 1.76.1
Author: B. Ellis, R. Gentleman, F. Hahne, N. Le Meur, D. Sarkar, M. Jiang
Maintainer: Mike Jiang <mike@ozette.com>
Description: Provides visualization tools for flow cytometry data.
Depends: R (>= 2.7.0), flowCore(>= 1.41.9), lattice
Imports: stats4, Biobase, flowCore, graphics, grDevices, grid,
        KernSmooth, lattice, latticeExtra, MASS, methods, RColorBrewer,
        stats, utils, hexbin,IDPmisc
Suggests: colorspace, flowStats, knitr, rmarkdown, markdown, testthat
License: Artistic-2.0
biocViews: ImmunoOncology, Infrastructure, FlowCytometry,
        CellBasedAssays, Visualization
VignetteBuilder: knitr
RoxygenNote: 6.1.1
git_url: https://git.bioconductor.org/packages/flowViz
git_branch: RELEASE_3_23
git_last_commit: eda6fc2
git_last_commit_date: 2026-08-25
Repository: Bioconductor 3.23
Date/Publication: 2026-08-26
NeedsCompilation: no
Packaged: 2026-08-26 21:50:01 UTC; biocbuild
