#' Visualization for flow cytometry
#' 
#' Functions and methods to visualize flow cytometry data. This package heavily
#' depends on the flowCore package.
#' 
#' \tabular{ll}{ Package: \tab flowViz\cr Type: \tab Package\cr Version: \tab
#' 0.2.1\cr Date: \tab 2006-11-16\cr License: \tab Artistic\cr } Traditionally,
#' large parts of the analysis process of flow cytometry data has been mostly
#' qualitative. To this end, dedicated visualization techniques have been used
#' for both quality control and infernece of the data. This package provides a
#' number of different visualization tools for flow data.
#' 
#' @name flowViz-package
#' @aliases flowViz-package flowViz
#' @docType package
#' @author
#' 
#' Maintainer: Florian Hahne <f.hahne@@dkfz.de> Authors: T. Duong, B. Ellis, R.
#' Gentleman, F. Hahne, N. Le Meur, D. Sarkar, M. Tang
#' @seealso \code{\link[flowCore:flowCore-package]{flowCore}}
#' @keywords package
#' @examples
#' 
#' ## examples go here
#' @import flowCore
#' @importClassesFrom methods ANY character formula logical matrix missing
#' @importMethodsFrom Biobase exprs pData pData<- phenoData sampleNames sampleNames<-
#' @importFrom methods as is
#' @importFrom grid gpar get.gpar grid.points grid.polygon grid.rect current.viewport unit
NULL
